import 'dart:collection';

import 'package:aliyun_oss_platform_interface/aliyun_oss_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:uuid/uuid.dart';

const MethodChannel _methodChannel =
    MethodChannel('com.wombat/aliyun_oss_plugin');

class MethodChannelAliyunOSS extends AliyunOSSPlatform {
  MethodChannelAliyunOSS() {
    _methodChannel.setMethodCallHandler(_methodCallHandler);
  }

  final Map<String, UploadMethodCallListener> _uploadMethodCallListeners = {};

  void _notifyUploadListeners(
      void Function(UploadMethodCallListener listener) callback) {
    _notifyListeners(callback, _uploadMethodCallListeners);
  }

  void _notifyListeners<T>(
      void Function(T listener) callback, Map<String, T> listeners) {
    listeners.forEach((key, value) {
      callback(value);
    });
  }

  _toMap(dynamic data) {
    return (data as Map<dynamic, dynamic>)
        .map((key, value) => MapEntry('$key', value));
  }

  Future<dynamic> _methodCallHandler(MethodCall methodCall) async {
    switch (methodCall.method) {

      /// 上传文件
      case 'upload_init':
        UploadInitData initData =
            UploadInitData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传初始化: ${initData.toJson()}");
        _notifyUploadListeners((listener) => listener.init?.call(initData));
        break;
      case "upload_progress":
        UploadProgressData progressData =
            UploadProgressData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传进度: ${progressData.toJson()}");
        _notifyUploadListeners(
            (listener) => listener.progress?.call(progressData));
        break;
      case "upload_success":
        UploadSuccessData successData =
            UploadSuccessData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传成功: ${successData.toJson()}");
        _notifyUploadListeners(
            (listener) => listener.success?.call(successData));
        break;
      case "upload_fail":
        UploadFailData failData =
            UploadFailData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传失败: ${failData.toJson()}");
        _notifyUploadListeners((listener) => listener.fail?.call(failData));
        break;
      case "upload_pause":
        _notifyUploadListeners((listener) =>
            listener.pause?.call(methodCall.arguments["objectKey"]));
        break;
      case "upload_resume":
        _notifyUploadListeners((listener) =>
            listener.resume?.call(methodCall.arguments["objectKey"]));
        break;
    }
  }

  @override
  Future<String> calculateMd5(String filePath) async {
    return (await _methodChannel
        .invokeMethod('calculate_md5', {'filePath': filePath})) as String;
  }

  @override
  initialize(String endpoint, String bucket, OSSCredentialProvider provider) {
    return _methodChannel.invokeMethod(
        'initialize',
        provider.toJson()
          ..addAll({
            "endpoint": endpoint,
            "bucket": bucket,
          }));
  }

  @override
  upload(String fileName, String filePath, String objectKey, String uuid) {
    return _methodChannel.invokeMethod('upload', {
      "fileName": fileName,
      "filePath": filePath,
      "objectKey": objectKey,
      "uuid": uuid,
    });
  }

  @override
  Future<String?> cancelUpload(String uuid, {bool delete = false}) {
    return _methodChannel.invokeMethod('cancel_upload', {
      'uuid': uuid,
      'delete': delete,
    });
  }

  @override
  String addUploadMethodCallListener(UploadMethodCallListener listener) {
    String uuid = Uuid().v4();
    _uploadMethodCallListeners[uuid] = listener;
    return uuid;
  }

  @override
  void removeUploadMethodCallListener({String? id, bool removeAll = false}) {
    if (removeAll) {
      _uploadMethodCallListeners.clear();
    } else {
      _uploadMethodCallListeners.remove(id);
    }
  }
}
